package loja.integration;

import javax.naming.*;
import loja.LojaException;

/**
 * $Revision: 1.3 $ $Date: 2003/05/25 15:53:02 $ by Helder da Rocha (helder@acm.org)
 * Encapsula servios de localizao JNDI. Este service locator  versatil
 * e pode ser usado para localizar objetos no Enterprise Naming Context (ENC)
 * via java:comp/env para chamadas dentro de containers e atravs do JNDI
 * global, para outros clientes, desde que o nome JNDI global seja igual ao
 * nome no ENC (exceo: Queue - tenta tambm default do servidor. 
 */
public class ServiceLocator {
	
	//JBoss standard connection factory & queue
	/** Fabrica de Conexoes default */
	public static final String JMS_DEFAULT_CONN_FACTORY  = "ConnectionFactory";
	
	private Context root, enc;
	
	// Implementao de Singleton
	private static ServiceLocator instance = null;
	private ServiceLocator() throws LojaException {
		try {
			root = new InitialContext();
		} catch (NamingException e) {
			throw new LojaException(e);
		}
	}
	
	/**
	 * Retorna a instancia desta classe.
	 */
	public static ServiceLocator getInstance() throws LojaException {
		if (instance == null) {
			instance = new ServiceLocator();
		}
		return instance;
	}
	
	/**
	 * Localizador generico para o ENC.
	 * Use para localizar uma variavel de ambiente (enviroment entry) do ENC (java:comp/env/nome) 
	 * declarada para um bean no ejb-jar.xml ou para o contexto no web.xml atravs do 
	 * tag &lt;env-entry&gt;.
	 */
	public Object findEnvEntry(String name) throws LojaException {
		try {
			enc  = (Context)root.lookup("java:comp/env");
			Object environmentReference = enc.lookup(name);
			return environmentReference;
		} catch (NamingException e) {
			throw new LojaException(e);
		}
	}
	
	/**
	 * Localiza um DataSource JDBC pelo nome local do ENC (java:comp/env/nome), 
	 * instancia uma conexo e a retorna.
	 */
	public java.sql.Connection findJdbcConnection(String name) throws LojaException {
		try {
			enc  = (Context)root.lookup("java:comp/env");
			javax.sql.DataSource dataSource = (javax.sql.DataSource) enc.lookup(name);
			return dataSource.getConnection();
		} catch (NamingException e) {
			throw new LojaException(e);
		} catch (java.sql.SQLException e) {
			throw new LojaException(e);
		}
	}
	
	/**
	 * Localiza a fbrica de conexes JMS default do servidor, instancia uma conexo 
	 * e a retorna. A fbrica default est na constante JMS_CONN_FACTORY e  ConnectionFactory
	 * no JBoss (tanto para queue como para topic).
	 */
	public javax.jms.QueueConnection findDefaultJMSQueueConnection() throws LojaException {
		try {
			javax.jms.QueueConnectionFactory factory = 
				(javax.jms.QueueConnectionFactory) root.lookup(JMS_DEFAULT_CONN_FACTORY);
			javax.jms.QueueConnection con = factory.createQueueConnection();
			return con;
		} catch (NamingException e) {
			throw new LojaException(e);
		} catch (javax.jms.JMSException e) {
			throw new LojaException(e);
		}
	}
	
	/**
	 * Localiza uma fila javax.jms.Queue.
	 * Este metodo tem dois fallbacks. Primeiro procura java:comp/env/nome. Se falhar, procura
	 * no contexto raiz usando nome da fila alternativa passada como segundo parametro.
	 */
	public javax.jms.Queue findQueue(String encName, String rootName) throws LojaException {
		Object ref;
		try {
			enc  = (Context)root.lookup("java:comp/env");
			ref = enc.lookup(encName);
		} catch (NamingException e) {
			System.out.println("Queue java:comp/env/" + encName + " nao encontrado no ENC... procurando no JNDI global!");
			try {
				ref = root.lookup(rootName);
			} catch (NamingException e2) {
				throw new LojaException("Queue "+rootName+" nao encontrado no JNDI global: " + e2);
			}
		}
		return (javax.jms.Queue) ref;
	}
	
	/**
	 * Localiza uma fila javax.jms.Queue.
	 * Este metodo tem dois fallbacks. Primeiro procura java:comp/env/nome. Se falhar, procura
	 * no contexto raiz usando mesmo nome.
	 */
	public javax.jms.Queue findQueue(String name) throws LojaException {
		return findQueue(name, name);
	}
	
	/**
	 * Localiza a interface Home de um EJB local (javax.ejb.EJBLocalHome).
	 * Este metodo tem dois fallbacks. Primeiro procura java:comp/env/nome. Se falhar, procura
	 * nome no contexto raiz.
	 */
	public javax.ejb.EJBLocalHome findLocalEJB(String name) throws LojaException {
		Object ref = null;
		try {
			enc  = (Context)root.lookup("java:comp/env");
			ref = enc.lookup(name);
		} catch (NamingException e) {
			System.out.println("Bean java:comp/env/" + name + " nao encontrado no ENC... procurando no JNDI global!");
			try {
				ref = root.lookup(name);
			} catch (NamingException e2) {
				throw new LojaException(e2);
			}
		}
		return (javax.ejb.EJBLocalHome) ref;
	}
	
	/**
	 * Localiza a interface Home de um EJB remoto (javax.ejb.EJBHome).  preciso passar a classe
	 * da interface como parmetro para realizar o narrow() obrigatrio em objetos remotos RMI-IIOP.
	 * Este metodo tem dois fallbacks. Primeiro procura java:comp/env/nome. Se falhar, procura
	 * nome no contexto raiz.
	 */
	public javax.ejb.EJBHome findRemoteEJB(String name, Class className) throws LojaException {
		Object ref = null;
		try {
			enc  = (Context)root.lookup("java:comp/env");
			ref = enc.lookup(name);
		} catch (NamingException e) {
			System.out.println("Bean  java:comp/env/" + name + " nao encontrado no ENC... procurando no JNDI global!");
			try {
				ref = root.lookup(name);
			} catch (NamingException e2) {
				throw new LojaException(e2);
			}
		}
		return (javax.ejb.EJBHome) javax.rmi.PortableRemoteObject.narrow(ref, className);
	}
			
}
