package loja.ejb;

import javax.ejb.*;
import javax.jms.*;
import java.util.*;

import loja.integration.*;
import loja.*;

public class ProcessoCompraMDB implements MessageDrivenBean, MessageListener {
	
	/**
	 * Fila do JBoss usada quando esta aplicao for chamada por
	 * clientes que nao usam o ENC (java:comp/env)
	 */
	private static final String FALLBACK_QUEUE_NAME   = "queue/C";
	
	private CompraLocalHome compraHome;
	private ClienteLocalHome clienteHome;
	
	private Queue outbox;
	private QueueConnection jmsCon;
	
	private MessageDrivenContext ctx;
	
	// Configurao da mensagem de e-mail.
	private String remetente, assuntoSucesso, assuntoFalha, mensagemSucesso, mensagemFalha;
	
	public void ejbCreate() {
		try {
			System.out.println("ProcessoCompraMDB: ejbCreate()");
			ServiceLocator locator = ServiceLocator.getInstance();
			jmsCon = locator.findDefaultJMSQueueConnection();
			
			// Localizacao dos beans usados por este bean
			compraHome =  (CompraLocalHome) locator.findLocalEJB("ejb/loja/compra");
			clienteHome = (ClienteLocalHome)locator.findLocalEJB("ejb/loja/cliente");
			
			// Fila que recebe mensagens de e-mail
			outbox = locator.findQueue("jms/loja/mailer", FALLBACK_QUEUE_NAME);

			// Localizao das variveis de ambiente do deployment descriptor (<env-entry>)
			remetente = (String) locator.findEnvEntry("email/remetente");
			assuntoSucesso 	= (String) locator.findEnvEntry("email/compra/concluida/assunto");
			assuntoFalha 	= (String) locator.findEnvEntry("email/compra/suspensa/assunto");
			mensagemSucesso = (String) locator.findEnvEntry("email/compra/concluida/mensagem");
			mensagemFalha 	= (String) locator.findEnvEntry("email/compra/suspensa/mensagem");
				
		} catch (LojaException e) {
			System.out.println("ProcessoCompraMDB-ejbCreate(): " + e);
		}
	}
	
	public void ejbRemove() {}
	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		System.out.println("setMessageDrivenContext()");
		this.ctx = ctx;
	}
	
	
	/**
	 * Aguarda mensagens com pedidos de compra.
	 */
	public void onMessage(Message message) {
		System.out.println("ProcessoCompraMDB: onMessage("+message+")");
		try {
			String compraId = message.getStringProperty("Compra");
			System.out.println("Processando compra " + compraId);
			CompraLocal compra = compraHome.findByPrimaryKey(new CompraPK(compraId));
			iniciar(compra);
		} catch (JMSException e) {
			System.out.println("ProcessoCompraMDB-onMessage(): " + e);
		} catch (FinderException e) {
			System.out.println("ProcessoCompraMDB-onMessage(): " + e);
		} catch (LojaException e) {
			System.out.println("ProcessoCompraMDB-onMessage(): " + e);
		} 
	}
	
	private void iniciar(CompraLocal compra) throws FinderException, LojaException {
		System.out.println("ProcessoCompraMDB: iniciar("+compra+")");
		// Recuperar cliente de compra
		ClienteLocal cliente = clienteHome.findByPrimaryKey(new ClientePK(compra.getClienteId()));
		
		// Verificar se tem credito usando administradora simulada.
		String cartao = cliente.getCartao();
		boolean temCredito = mundo.Administradora.verificarCartao(cartao);

		if (temCredito) {
			if (compra.getStatus() == Status.SUSPENSO) { // compra estava suspensa
				compra.setStatus(Status.EM_ANDAMENTO);
			}
			if (compra.getStatus() == Status.EM_ANDAMENTO) {
				processar(compra, cliente);
			} else {
				System.out.println("ProcessoCompraMDB: Estado invalido para pedido de compra: " + compra.getStatus());
			}
		} else {
			compra.setStatus(Status.SUSPENSO);
			System.out.println("Credito rejeitado! Pedido suspenso.");
			try {
				enviarEmail(cliente.getEmail(), assuntoFalha, mensagemFalha);
			} catch (JMSException e) {
				System.out.println("E-mail nao foi enviado: " + e);
			}
		}
	}
	
	private void processar(CompraLocal compra, ClienteLocal cliente) throws LojaException {
		System.out.println("ProcessoCompraMDB: processar("+compra+","+cliente+")");
		// Realizar compra
		String nota = comprar(compra);
		compra.setStatus(Status.CONCLUIDO);		
		System.out.println("PEDIDO PROCESSADO: \n" + nota);
		
		try {
			enviarEmail(cliente.getEmail(), assuntoSucesso, mensagemSucesso + "\n" + nota);
		} catch (JMSException e) {
			System.out.println("E-mail nao foi enviado: " + e);
		}
	}
	
	private String comprar(CompraLocal compra) throws LojaException {
		System.out.println("ProcessoCompraMDB: comprar("+compra+")");

		// 1. Reduzir estoque
		ItemLocal[] itens = compra.getItens();
		for (int i = 0; i < itens.length; i++) {
			ProdutoLocal p = itens[i].getProduto();
			int qte = itens[i].getQuantidade();
			int estoque = p.getQuantidade();
			if (qte <= estoque) {
				p.removeItens(qte);
			} else { // isto nao deve acontecer - use transacoes para evitar acesso concorrente!
				compra.setStatus(Status.IRREGULAR);
				System.out.println("Estoque foi reduzido durante a transacao e nao pode atender o pedido!");
			}
		}
		// 2. Enviar recibo
		return gerarNota("Nota de Compra", compra);
	}
	
	private String gerarNota(String titulo, CompraLocal compra) {
		String it = "";
		ItemLocal[] itens = compra.getItens();
		for (int i = 0; i < itens.length; i++) {
			it += "\n" + itens[i].getQuantidade() + " unidades de " + 
			             itens[i].getProduto().getNome() + ", (cod. " +
						 itens[i].getProduto().getCodigo() + ") por $" +
						 itens[i].getProduto().getPreco() + " cada. Sub-total: $" +
						 itens[i].getSubTotal();
		}
		return 	"[" + titulo + "] " + 
				compra.getClienteId() + ": " + 
				compra.getNumero() + " TOTAL da Nota $" + 
				compra.getTotal() +
				it;
	}
	
	/**
	 * Encaminha mensagem com dados que devem ser enviados por e-mail ao MDB MailerDB,
	 * que aguarda mensagens na fila.
	 */
	private void enviarEmail(String email, String assunto, String mensagem) throws JMSException {
		System.out.println("ProcessoCompraMDB: enviarEmail("+email+","+assunto+","+mensagem+")");
		QueueSession session = jmsCon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		TextMessage message = session.createTextMessage(mensagem);
		message.setStringProperty("Assunto", assunto);
		message.setStringProperty("Para", email);
		message.setStringProperty("De", remetente);
		QueueSender sender = session.createSender( outbox );
		sender.send(message);
	}
	
}
